@@ -4,7 +4,7 @@
 #  | .__/  \___| \___||_|\_\
 #  |_| like print, but easy.
 
-__version__ = "24.0.1"
+__version__ = "24.0.5"
 
 """
 See https://github.com/salabim/peek for details
@@ -42,6 +42,7 @@ if Pythonista:
     import clipboard
 else:
     import colorama
+
     colorama.just_fix_windows_console()
     import pyperclip
 
@@ -55,6 +56,7 @@ def perf_counter():
     return time.perf_counter() if _fixed_perf_counter is None else _fixed_perf_counter
 
 
+codes = {}
 colors = dict(
     black="\033[0;30m",
     red="\033[0;31m",
@@ -66,6 +68,7 @@ colors = dict(
     white="\033[0;37m",
 )
 colors["-"] = "\033[0m"
+colors[""] = "\033[0m"
 
 
 ansi_to_rgb = {
@@ -110,8 +113,6 @@ def check_validity(name, value):
             return
 
     if name in ("color", "color_value"):
-        if name == "color_value" and value == "":
-            return
         if isinstance(value, str) and value.lower() in colors:
             return
 
@@ -119,13 +120,6 @@ def check_validity(name, value):
             if isinstance(value, numbers.Number):
                 return
 
-    elif name == "enforce_line_length":
-        if value:
-            if isinstance(value, numbers.Number) and value > 0:
-                return
-        else:
-            return
-
     elif name == "line_length":
         if isinstance(value, numbers.Number) and value > 0:
             return
@@ -160,27 +154,14 @@ def check_validity(name, value):
     raise AttributeError(f"incorrect {name_org}: {repr(value)}{in_read_toml_message}")
 
 
-class Source(executing.Source):
-    def get_text_with_indentation(self, node):
-        result = self.asttokens().get_text(node)
-        if "\n" in result:
-            result = " " * node.first_token.start[1] + result
-            result = textwrap.dedent(result)
-        result = result.strip()
-        return result
-
-
-def change_path(new_path):  # used in tests
-    global Path
-    Path = new_path
-
-
 _fixed_perf_counter = None
 
 
-def spec_to_attributes(d):
+def spec_to_attributes(**kwargs):
     result = {}
-    for name, value in d.items():
+    for name, value in kwargs.items():
+        if alias_name.get(name, "") in kwargs:
+            raise AttributeError(f"not allowed to use {name} and {alias_name.get(name)} both.")
         check_validity(name, value)
         name = de_alias(name)
         if name == "delta" and value is not None:
@@ -202,6 +183,19 @@ def read_toml():
     return {}
 
 
+def print_pythonista_color(s, end="\n"):
+    while s:
+        for ansi, rgb in ansi_to_rgb.items():
+            if s.startswith(ansi):
+                console.set_color(*rgb)
+                s = s[len(ansi) :]
+                break
+        else:
+            print(s[0], end="")
+            s = s[1:]
+    print("", end=end)
+
+
 def no_source_error():
     raise NotImplementedError(
         """
@@ -224,7 +218,7 @@ def return_args(args, return_none):
 
 class _Peek:
     def __init__(self, parent=None, **kwargs):
-        self._attributes = spec_to_attributes(kwargs)
+        self._attributes = spec_to_attributes(**kwargs)
         self._parent = parent
 
     def new(self, ignore_toml=False, **kwargs):
@@ -238,11 +232,11 @@ class _Peek:
 
     def clone(self, **kwargs):
         clone = _Peek(parent=self._parent)
-        clone._attributes = {**self._attributes, **spec_to_attributes(kwargs)}
+        clone._attributes = {**self._attributes, **spec_to_attributes(**kwargs)}
         return clone
 
     def configure(self, **kwargs):
-        self._attributes.update(spec_to_attributes(kwargs))
+        self._attributes.update(spec_to_attributes(**kwargs))
 
     def __getattr__(self, item, spec=False):
         item = de_alias(item)
@@ -254,15 +248,18 @@ class _Peek:
                 return perf_counter() - node._attributes["perf_counter"] + node._attributes["delta"]
             elif item == "perf_counter":
                 return node._attributes["delta"]
+            elif item == "prefix":
+                prefix = node._attributes[item]
+                return str(prefix() if callable(prefix) else prefix)
             else:
                 return node._attributes[item]
         else:
             return self.__getattribute__(item)
 
     def __setattr__(self, item, value):
-        if item in ("_parent", "is_context_manager", "line_number_with_filename_and_parent", "save_traceback", "enter_time", "as_str", "_attributes"):
+        if item in ("_parent", "_is_context_manager", "_line_number_with_filename_and_parent", "_save_traceback", "_enter_time", "_as_str", "_attributes"):
             return super().__setattr__(item, value)
-        self._attributes.update(spec_to_attributes({item: value}))
+        self._attributes.update(spec_to_attributes(**{item: value}))
 
     def __repr__(self):
         pairs = [
@@ -286,18 +283,41 @@ class _Peek:
                 return False
         return self.enabled
 
+    def print(self, *args, as_str=False, **kwargs):
+        print_like=True
+        if "print" in kwargs and "print_like" in kwargs:
+            raise AttributeError("both print_like and print specified")
+        print_like=kwargs.get["print_like", print_like]
+        print_like=kwargs.get["print", print_like]
+        kwargs["print_like"]=print_like
+        return self(*args, as_str=as_str, **kwargs)
+
     def __call__(self, *args, as_str=False, **kwargs):
-        codes = {}
         this = self.fork(**kwargs)
 
-        this.as_str = as_str
+        this._as_str = as_str
+
+        if this.print_like:
+            if "sep" in kwargs or "separator" in kwargs:
+                if "sepp" in kwargs or "separator_print" in kwargs:
+                    raise AttributeError("both separator and separator_print specified")
+                this.separator_print = this.separator
+
+            this.values_only = True
+            this.show_enter = False
+            this.show_exit = False
+            this.show_traceback = False
+            this.to_clipboard = False
+            this.return_none = True
+            args = [this.separator_print.join(map(str, args))]
+
         if len(args) != 0 and not this.do_show():
             if as_str:
                 return ""
             else:
                 return return_args(args, this.return_none)
 
-        self.is_context_manager = False
+        self._is_context_manager = False
 
         Pair = collections.namedtuple("Pair", "left right")
 
@@ -340,7 +360,7 @@ class _Peek:
             frame_info = inspect.getframeinfo(call_frame, context=1)
 
             #            parent_function = frame_info.function
-            parent_function = Source.executing(call_frame).code_qualname()  # changed in version 1.3.10 to include class name
+            parent_function = executing.Source.executing(call_frame).code_qualname()  # changed in version 1.3.10 to include class name
             parent_function = parent_function.replace(".<locals>.", ".")
             if parent_function == "<module>" or str(this.show_line_number) in ("n", "no parent"):
                 parent_function = ""
@@ -355,7 +375,7 @@ class _Peek:
                 this_line_prev = code[line_number - 2].strip()
             else:
                 this_line_prev = ""
-        if this_line.startswith("@") or this_line_prev.startswith("@"):
+        if (this_line.startswith("@") or this_line_prev.startswith("@")) and (this.show_enter or this.show_exit):
             if as_str:
                 raise TypeError("as_str may not be True when peek used as decorator")
 
@@ -365,7 +385,7 @@ class _Peek:
                     break
             else:
                 line_number += 1
-            this.line_number_with_filename_and_parent = f"#{line_number}{filename_name}{parent_function}"
+            this._line_number_with_filename_and_parent = f"#{line_number}{filename_name}{parent_function}"
 
             def real_decorator(function):
                 @functools.wraps(function)
@@ -395,23 +415,23 @@ class _Peek:
             return real_decorator
 
         if filename in ("<stdin>", "<string>"):
-            this.line_number_with_filename_and_parent = ""
+            this._line_number_with_filename_and_parent = ""
         else:
-            call_node = Source.executing(call_frame).node
+            call_node = executing.Source.executing(call_frame).node
             if call_node is None:
                 no_source_error()
             line_number = call_node.lineno
             this_line = code[line_number - 1].strip()
 
-            this.line_number_with_filename_and_parent = f"#{line_number}{filename_name}{parent_function}"
+            this._line_number_with_filename_and_parent = f"#{line_number}{filename_name}{parent_function}"
 
-        if this_line.startswith("with ") or this_line.startswith("with\t"):
+        if (this_line.startswith("with ") or this_line.startswith("with\t")) and (this.show_enter or this.show_exit):
             if as_str:
                 raise TypeError("as_str may not be True when peek used as context manager")
             if args:
                 raise TypeError("non-keyword arguments are not allowed when peek used as context manager")
 
-            this.is_context_manager = True
+            this._is_context_manager = True
             return this
 
         if not this.do_show():
@@ -426,7 +446,7 @@ class _Peek:
                 for right in args:
                     pairs.append(Pair(left="", right=right))
             else:
-                source = Source.for_frame(call_frame)
+                source = executing.Source.for_frame(call_frame)
                 for node, right in zip(call_node.args, args):
                     left = source.asttokens().get_text(node)
                     if "\n" in left:
@@ -456,6 +476,7 @@ class _Peek:
             if not (len(pairs) > 1 and this.separator == ""):
                 if not any("\n" in pair.left for pair in pairs):
                     as_one_line = context + this.separator.join(pair.left + this.serialize_kwargs(obj=pair.right, width=10000) for pair in pairs)
+                    #                    as_one_line = context + this.separator.join(pair.left + (this.serialize_kwargs(obj=pair.right, width=10000)) for pair in pairs)
                     if len(as_one_line) <= this.line_length and "\n" not in as_one_line:
                         out = as_one_line
                         just_one_line = True
@@ -513,9 +534,7 @@ class _Peek:
 
         if as_str:
             if this.do_show():
-                if this.enforce_line_length:
-                    out = "\n".join(line[: this.line_length] for line in out.splitlines())
-                return out + "\n"
+                return out + this.end
             else:
                 return ""
         if this.to_clipboard:
@@ -531,25 +550,25 @@ class _Peek:
         self._attributes = save
 
     def __enter__(self):
-        if not hasattr(self, "is_context_manager"):
+        if not hasattr(self, "_is_context_manager"):
             raise ValueError("not allowed as context_manager")
-        self.save_traceback = self.traceback()
-        self.enter_time = perf_counter()
+        self._save_traceback = self.traceback()
+        self._enter_time = perf_counter()
         if self.show_enter:
             context = self.context()
-            self.do_output(context + "enter" + self.save_traceback)
+            self.do_output(context + "enter" + self._save_traceback)
         return self
 
     def __exit__(self, *args):
         if self.show_exit:
             context = self.context()
-            duration = perf_counter() - self.enter_time
-            self.do_output(f"{context}exit in {duration:.6f} seconds{self.save_traceback}")
-        self.is_context_manager = False
+            duration = perf_counter() - self._enter_time
+            self.do_output(f"{context}exit in {duration:.6f} seconds{self._save_traceback}")
+        self._is_context_manager = False
 
-    def context(self, omit_context_separator=False):
-        if self.show_line_number and self.line_number_with_filename_and_parent != "":
-            parts = [self.line_number_with_filename_and_parent]
+    def context(self, omit_line_number=False, omit_context_separator=False):
+        if not omit_line_number and self.show_line_number and self._line_number_with_filename_and_parent != "":
+            parts = [self._line_number_with_filename_and_parent]
         else:
             parts = []
         if self.show_time:
@@ -562,10 +581,10 @@ class _Peek:
         if not omit_context_separator and context:
             context += self.context_separator
 
-        return str(self.prefix() if callable(self.prefix) else self.prefix) + context
+        return self.prefix + context
 
     def add_color_value(self, s):
-        if self.output != "stdout" or self.as_str:
+        if self.output != "stdout" or self._as_str:
             return s
         if self.color_value.lower() not in (self.color.lower(), ""):
             return colors[self.color_value.lower()] + s + colors[self.color.lower()]
@@ -573,33 +592,26 @@ class _Peek:
             return s
 
     def do_output(self, s):
-        if self.enforce_line_length:
-            s = "\n".join(line[: self.line_length] for line in s.splitlines())
         if self.do_show():
             if callable(self.output):
-                self.output(s)
-            elif self.output == "stderr":
-                print(s, file=sys.stderr)
-            elif self.output == "stdout":
-                if self.color != "-" or self.color_value != "-":
-                    s = colors[self.color.lower()] + s + colors["-"]
-                    if Pythonista:
-                        while s:
-                            for ansi, rgb in ansi_to_rgb.items():
-                                if s.startswith(ansi):
-                                    console.set_color(*rgb)
-                                    s = s[len(ansi) :]
-                                    break
-                            else:
-                                print(s[0], end="")
-                                s = s[1:]
-                        print()
-                    else:
-                        print(s)
+                if self.output == builtins.print or "end" in inspect.signature(self.output).parameters:
+                    # test for builtins.print is required as for Python <= 3.10, builtins.print has no signature
+                    self.output(s, end=self.end)
                 else:
-                    print(s)
+                    self.output(s + self.end)
+            elif self.output == "stderr":
+                print(s, file=sys.stderr, end=self.end)
+            elif self.output == "stdout":
+                if self.color not in ["", "-"]:
+                    s = colors[self.color.lower()] + s + self.end + colors["-"]
+                    if Pythonista:
+                        print_pythonista_color(s, end="")
+                    else:
+                        print(s, end="")
+                else:
+                    print(s, end=self.end)
             elif self.output == "stdout_nocolor":
-                print(s)
+                print(s, end=self.end)
             elif self.output == "logging.debug":
                 logging.debug(s)
             elif self.output == "logging.info":
@@ -614,12 +626,12 @@ class _Peek:
                 pass
             elif isinstance(self.output, str):
                 with open(self.output, "a+", encoding="utf-8") as f:
-                    print(s, file=f)
+                    print(s, file=f, end=self.end)
             elif isinstance(self.output, Path):
                 with self.output.open("a+", encoding="utf-8") as f:
-                    print(s, file=f)
+                    print(s, file=f, end=self.end)
             else:
-                print(s, file=self.output)
+                print(s, file=self.output, end=self.end)
 
     def copy_to_clipboard(self, value, confirm=True):
         if Pythonista:
@@ -646,8 +658,9 @@ class _Peek:
             return ""
 
     def serialize_kwargs(self, obj, width):
-        if isinstance(obj, str) and not self.quote_string:
-            return self.add_color_value(obj)
+        if isinstance(obj, str):
+            if not self.quote_string:
+                return str(self.add_color_value(obj))
         kwargs = {
             key: getattr(self, key)
             for key in ("sort_dicts", "compact", "indent", "depth", "underscore_numbers")
@@ -661,6 +674,7 @@ class _Peek:
 
 store_perf_counter = perf_counter()
 name_alias_default = (
+    # name, alias, default value
     ("color", "col", "-"),
     ("color_value", "col_val", ""),
     ("compact", "", False),
@@ -668,7 +682,7 @@ name_alias_default = (
     ("delta", "", 0),
     ("depth", "", 1000000),
     ("enabled", "", True),
-    ("enforce_line_length", "", False),
+    ("end", "", "\n"),
     ("equals_separator", "", "="),
     ("filter", "f", ""),
     ("indent", "", 1),
@@ -676,9 +690,11 @@ name_alias_default = (
     ("line_length", "ll", 80),
     ("output", "", "stdout"),
     ("prefix", "pr", ""),
+    ("print_like", "print", False),
     ("quote_string", "qs", True),
     ("return_none", "", False),
     ("separator", "sep", ", "),
+    ("separator_print", "sepp", " "),
     ("serialize", "", pprint.pformat),
     ("show_delta", "sd", False),
     ("show_enter", "se", True),
@@ -694,6 +710,7 @@ name_alias_default = (
     ("wrap_indent", "", "    "),
 )
 alias_name = {alias: name for (name, alias, default) in name_alias_default if alias}
+name_alias = {name: alias for (name, alias, default) in name_alias_default}
 name_default = {name: default for (name, alias, default) in name_alias_default}
 alias_default = {alias: default for (name, alias, default) in name_alias_default if alias}
 name_and_alias_default = {**name_default, **alias_default}
@@ -705,6 +722,7 @@ in_read_toml_message = ""
 peek = peek_toml.new()
 builtins.peek = peek
 
+
 class PeekModule(types.ModuleType):
     def __call__(self, *args, **kwargs):
         return peek(*args, **kwargs)
