@@ -301,9 +301,17 @@ def test_print(capsys):
 2
 """
     )
+    result=peek.print(1,2,as_str=True)
+    assert result=="1 2\n"
+
+    result=peek.print(1,2)
+    assert result is None
+
+
     with pytest.raises(AttributeError):
         peek.print(sep="|", sepp="/")   
 
+
 def test_clone():
     hello = "world"
     z = peek.clone()
@@ -449,7 +457,7 @@ def test_decorator(capsys):
         return x - y
 
     @peek(show_enter=False, show_exit=False)
-    def pos(x, y):
+    def pow(x, y):
         return x**y
 
     assert div(10, 2) == 10 / 2
